import { FetchRequest, FetchOptions, FetchResponse } from 'ohmyfetch';
import { Preset } from 'unenv';
import { Unimport, UnimportOptions } from 'unimport';
import { PluginVisualizerOptions } from 'rollup-plugin-visualizer';
import { Hookable, NestedHooks } from 'hookable';
import { Consola, LogLevel } from 'consola';
import { WatchOptions, FSWatcher } from 'chokidar';
import { NodeFileTraceOptions } from '@vercel/nft';
import { InputOptions, OutputOptions } from 'rollup';
import { Loader } from 'esbuild';
import { FilterPattern } from '@rollup/pluginutils';
import { Handler, App } from 'h3';
import { Worker } from 'worker_threads';
import { ListenOptions, Listener } from 'listhen';

interface InternalApi {
}
declare type ValueOf<C> = C extends Record<any, any> ? C[keyof C] : never;
declare type MatchedRoutes<Route extends string> = ValueOf<{
    [key in keyof InternalApi]: Route extends key | `${key}/${string}` | '/' ? key : never;
}>;
declare type MiddlewareOf<Route extends string> = Exclude<InternalApi[MatchedRoutes<Route>], Error | void>;
declare type TypedInternalResponse<Route, Default> = Default extends string | boolean | number | null | void | object ? Default : Route extends string ? MiddlewareOf<Route> extends never ? Default : MiddlewareOf<Route> : Default;
interface $Fetch<DefaultT = unknown, DefaultR extends FetchRequest = FetchRequest> {
    <T = DefaultT, R extends FetchRequest = DefaultR>(request: R, opts?: FetchOptions): Promise<TypedInternalResponse<R, T>>;
    raw<T = DefaultT, R extends FetchRequest = DefaultR>(request: R, opts?: FetchOptions): Promise<FetchResponse<TypedInternalResponse<R, T>>>;
}
declare global {
    var $fetch: $Fetch;
    namespace NodeJS {
        interface Global {
            $fetch: $Fetch;
        }
    }
}

interface NodeExternalsOptions {
    inline?: string[];
    external?: string[];
    outDir?: string;
    trace?: boolean;
    traceOptions?: NodeFileTraceOptions;
    moduleDirectories?: string[];
    exportConditions?: string[];
    traceInclude?: string[];
}

interface StorageMounts {
    [path: string]: {
        driver: 'fs' | 'http' | 'memory' | 'redis' | 'cloudflare-kv';
        [option: string]: any;
    };
}

declare type RollupConfig = InputOptions & {
    output: OutputOptions;
};

declare type Options = {
    include?: FilterPattern;
    exclude?: FilterPattern;
    sourceMap?: boolean;
    minify?: boolean;
    target?: string | string[];
    jsxFactory?: string;
    jsxFragment?: string;
    define?: {
        [k: string]: string;
    };
    /**
     * Use this tsconfig file instead
     * Disable it by setting to `false`
     */
    tsconfig?: string | false;
    /**
     * Map extension to esbuild loader
     * Note that each entry (the extension) needs to start with a dot
     */
    loaders?: {
        [ext: string]: Loader | false;
    };
};

interface NitroEventHandler {
    /**
     * Path prefix or route
     *
     * If an empty string used, will be used as a middleware
     */
    route?: string;
    /**
     * Use lazy loading to import handler
     */
    lazy?: boolean;
    /**
     * Path to event handler
     *
     */
    handler: string;
    /**
     * Router method matcher
     */
    method?: string;
}
interface NitroDevEventHandler {
    /**
     * Path prefix or route
     */
    route?: string;
    /**
     * Event handler
     *
     */
    handler: Handler;
}

interface Nitro {
    options: NitroOptions;
    scannedHandlers: NitroEventHandler[];
    vfs: Record<string, string>;
    hooks: Hookable<NitroHooks>;
    unimport?: Unimport;
    logger: Consola;
    close: () => Promise<void>;
}
declare type HookResult = void | Promise<void>;
interface NitroHooks {
    'nitro:rollup:before': (nitro: Nitro) => HookResult;
    'nitro:compiled': (nitro: Nitro) => HookResult;
    'nitro:dev:reload': () => HookResult;
    'close': () => HookResult;
}
declare type DeepPartial<T> = T extends Record<string, any> ? {
    [P in keyof T]?: DeepPartial<T[P]> | T[P];
} : T;
declare type NitroPreset = NitroConfig | (() => NitroConfig);
interface NitroConfig extends DeepPartial<NitroOptions> {
    extends?: string | string[] | NitroPreset;
}
interface NitroRouteOption {
    swr?: boolean | number;
    redirect?: string;
}
interface NitroRoutesOptions {
    [path: string]: NitroRouteOption;
}
interface PublicAssetDir {
    baseURL?: string;
    fallthrough?: boolean;
    maxAge: number;
    dir: string;
}
interface ServerAssetDir {
    baseName: string;
    dir: string;
}
interface DevServerOptions {
    watch: string[];
}
interface NitroOptions {
    _config: NitroConfig;
    preset: string;
    logLevel: LogLevel;
    runtimeConfig: {
        app: {
            baseURL: string;
        };
        nitro: {
            /** @deprecated Use top-level routes option! */
            routes: NitroRoutesOptions;
        };
        [key: string]: any;
    };
    rootDir: string;
    srcDir: string;
    scanDirs: string[];
    buildDir: string;
    output: {
        dir: string;
        serverDir: string;
        publicDir: string;
    };
    storage: StorageMounts;
    timing: boolean;
    renderer: string;
    serveStatic: boolean;
    experimental?: {
        wasm?: boolean;
    };
    serverAssets: ServerAssetDir[];
    publicAssets: PublicAssetDir[];
    autoImport: UnimportOptions;
    plugins: string[];
    virtual: Record<string, string | (() => string | Promise<string>)>;
    dev: boolean;
    devServer: DevServerOptions;
    watchOptions: WatchOptions;
    baseURL: string;
    handlers: NitroEventHandler[];
    routes: NitroRoutesOptions;
    devHandlers: NitroDevEventHandler[];
    prerender: {
        crawlLinks: boolean;
        routes: string[];
    };
    rollupConfig?: RollupConfig;
    entry: string;
    unenv: Preset;
    alias: Record<string, string>;
    minify: boolean;
    inlineDynamicImports: boolean;
    sourceMap: boolean;
    node: boolean;
    moduleSideEffects: string[];
    esbuild?: {
        options?: Options;
    };
    externals: boolean | NodeExternalsOptions;
    analyze: false | PluginVisualizerOptions;
    replace: Record<string, string | ((id: string) => string)>;
    typescript: {
        generateTsConfig?: boolean;
    };
    hooks: NestedHooks<NitroHooks>;
    nodeModulesDirs: string[];
    commands: {
        preview: string;
        deploy: string;
    };
}

declare function prepare(nitro: Nitro): Promise<void>;
declare function copyPublicAssets(nitro: Nitro): Promise<void>;
declare function build(nitro: Nitro): Promise<void | {
    entry: string;
}>;
declare function writeTypes(nitro: Nitro): Promise<void>;

declare function createNitro(config?: NitroConfig): Promise<Nitro>;

declare const GLOB_SCAN_PATTERN = "**/*.{ts,mjs,js,cjs}";
declare type FileInfo = {
    dir: string;
    path: string;
    fullPath: string;
};
declare function scanHandlers(nitro: Nitro): Promise<{
    dirs: string[];
    files: FileInfo[];
    handlers: NitroEventHandler[];
}[]>;
declare function scanMiddleware(nitro: Nitro): Promise<{
    dirs: string[];
    files: FileInfo[];
    handlers: NitroEventHandler[];
}>;
declare function scanRoutes(nitro: Nitro, dir: string, prefix?: string): Promise<{
    dirs: string[];
    files: FileInfo[];
    handlers: NitroEventHandler[];
}>;

interface NitroWorker {
    worker: Worker;
    address: {
        host: string;
        port: number;
        socketPath?: string;
    };
}
interface NitroDevServer {
    reload: () => void;
    listen: (port: ListenOptions['port'], opts?: Partial<ListenOptions>) => Promise<Listener>;
    app: App;
    close: () => Promise<void>;
    watcher?: FSWatcher;
}
declare function createDevServer(nitro: Nitro): NitroDevServer;

declare function loadOptions(userConfig?: NitroConfig): Promise<NitroOptions>;
declare function defineNitroConfig(config: NitroConfig): NitroConfig;

declare function prerender(nitro: Nitro): Promise<void>;

declare function defineNitroPreset(preset: NitroPreset): NitroPreset;

export { $Fetch, DevServerOptions, GLOB_SCAN_PATTERN, InternalApi, MatchedRoutes, MiddlewareOf, Nitro, NitroConfig, NitroDevEventHandler, NitroDevServer, NitroEventHandler, NitroHooks, NitroOptions, NitroPreset, NitroRouteOption, NitroRoutesOptions, NitroWorker, PublicAssetDir, ServerAssetDir, TypedInternalResponse, ValueOf, build, copyPublicAssets, createDevServer, createNitro, defineNitroConfig, defineNitroPreset, loadOptions, prepare, prerender, scanHandlers, scanMiddleware, scanRoutes, writeTypes };
